document.addEventListener('DOMContentLoaded', () => {
    const app = new ProcessDolphin();
});

class ProcessDolphin {
    constructor() {
        this.isRunning = false;
        this.startTime = null;
        this.runtimeInterval = null;
        this.updateDataInterval = null;
        this.processData = {};
        this.activeProcessId = null;
        this.activeDirection = null;
        this.activeTab = 'processes';
        this._firstUpdate = true;
        this._processListCache = null;

        this.runButton = document.getElementById('run-button');
        this.exportButton = document.getElementById('export-button');
        this.processList = document.getElementById('process-list');
        this.processSearch = document.getElementById('process-search');
        this.detailPanel = document.getElementById('detail-panel');
        this.closeDetailButton = document.getElementById('close-detail');
        this.runtimeElement = document.getElementById('runtime');
        this.processCountElement = document.getElementById('process-count');
        this.tabButtons = document.querySelectorAll('.tab-button');
        this.tabContents = document.querySelectorAll('.tab-content');

        this.preloadFontAwesome();

        this.initEventListeners();

        this.render();
    }

    preloadFontAwesome() {
        const icons = [
            'fa-play', 'fa-stop', 'fa-spinner', 'fa-file-export',
            'fa-microchip', 'fa-network-wired', 'fa-globe',
            'fa-search', 'fa-terminal', 'fa-arrow-right-to-bracket',
            'fa-arrow-right-from-bracket', 'fa-xmark', 'fa-check-circle',
            'fa-exclamation-circle', 'fa-folder', 'fa-file-alt',
            'fa-code', 'fa-cogs'
        ];

        const div = document.createElement('div');
        div.style.position = 'absolute';
        div.style.visibility = 'hidden';
        div.style.width = '0';
        div.style.height = '0';
        div.style.overflow = 'hidden';

        icons.forEach(icon => {
            const i = document.createElement('i');
            i.className = `fa-solid ${icon}`;
            div.appendChild(i);
        });

        document.body.appendChild(div);

        setTimeout(() => {
            div.remove();
        }, 2000);
    }

    initEventListeners() {
        this.runButton.addEventListener('click', () => this.toggleRunning());

        this.exportButton.addEventListener('click', () => this.exportData());

        this.processSearch.addEventListener('input', () => this.handleSearchInput());
        this.closeDetailButton.addEventListener('click', () => this.closeDetailPanel());

        this.tabButtons.forEach(button => {
            button.addEventListener('click', () => {
                const tabName = button.getAttribute('data-tab');
                this.switchTab(tabName);
            });
        });

        document.addEventListener('keydown', (e) => {
            if (e.key === 'Escape') {
                this.closeDetailPanel();
            }
        });
    }

    switchTab(tabName) {
        this.activeTab = tabName;

        this.tabButtons.forEach(button => {
            if (button.getAttribute('data-tab') === tabName) {
                button.classList.add('active');
            } else {
                button.classList.remove('active');
            }
        });

        this.tabContents.forEach(content => {
            if (content.id === `${tabName}-tab`) {
                content.classList.add('active');
            } else {
                content.classList.remove('active');
            }
        });
    }

    async toggleRunning() {
        if (!this.isRunning) {
            this.runButton.innerHTML = '<i class="fa-solid fa-spinner fa-spin"></i> Starting...';
            this.runButton.disabled = true;

            const emptyState = document.querySelector('.process-list-empty');
            if (emptyState) {
                emptyState.innerHTML = `
                    <i class="fa-solid fa-spinner fa-spin"></i>
                    <p>Scanning for network processes...</p>
                    <span>This will only take a moment</span>
                `;
            }
        }

        const response = await window.pywebview.api.toggle_monitoring();

        if (response.status === 'success') {
            this.isRunning = !this.isRunning;

            if (this.isRunning) {
                this.runtimeInterval = setInterval(() => this.updateRuntime(), 1000);

                await this.updateData();

                this.updateDataInterval = setInterval(() => this.updateData(), 1000);

                this.runButton.innerHTML = '<i class="fa-solid fa-stop"></i> Stop';
                this.runButton.classList.add('running');
                document.querySelector('.app-container').classList.add('running');

                const emptyState = document.querySelector('.process-list-empty');
                if (emptyState) {
                    emptyState.remove();
                }
            } else {
                clearInterval(this.runtimeInterval);
                clearInterval(this.updateDataInterval);
                this.runButton.innerHTML = '<i class="fa-solid fa-play"></i> Run';
                this.runButton.classList.remove('running');
                document.querySelector('.app-container').classList.remove('running');

                if (Object.keys(this.processData).length > 0) {
                    this.exportButton.disabled = false;
                }
            }

            this.runButton.disabled = false;
        } else {
            this.runButton.innerHTML = '<i class="fa-solid fa-play"></i> Run';
            this.runButton.disabled = false;
        }
    }

    async updateRuntime() {
        const runtime = await window.pywebview.api.get_runtime();
        this.runtimeElement.textContent = runtime;
    }

    async updateProcessCount() {
        const count = await window.pywebview.api.get_process_count();
        this.processCountElement.textContent = count.toString();
    }

    async updateData() {
        try {
            const dataPromise = window.pywebview.api.get_process_data();

            const timeoutPromise = new Promise((_, reject) => {
                setTimeout(() => reject(new Error('Data fetch timeout')), 2000);
            });

            const data = await Promise.race([dataPromise, timeoutPromise]);

            this.processData = data;

            // Immediately highlight processes with external connections
            if (this.isRunning) {
                this.updateExternalConnectionHighlights();
            }

            if (this.activeProcessId && this.activeDirection) {
                this.showProcessDetails(this.activeProcessId, this.activeDirection);
            }

            this.updateProcessCount();

            if (this.activeTab === 'processes') {
                this.renderProcessList();
            }

            if (this._firstUpdate) {
                this._firstUpdate = false;
            }
        } catch (error) {
            console.error('Error updating data:', error);
        }
    }
    
    updateExternalConnectionHighlights() {
        // Get all process items
        const processItems = document.querySelectorAll('.process-item');
        
        processItems.forEach(item => {
            const processId = item.dataset.processId;
            if (!processId || !this.processData[processId]) return;
            
            const process = this.processData[processId];
            
            // Check if process has an external connection
            if (process.outbound && 
                process.outbound.lastActivity && 
                !this.isLocalhostIP(process.outbound.dstIp) && 
                this.getIPType(process.outbound.dstIp) === 'external') {
                
                // Add external-connection class if it doesn't already have it
                if (!item.classList.contains('external-connection')) {
                    item.classList.add('external-connection');
                    
                    // Add globe icon if it doesn't exist
                    if (!item.querySelector('.external-indicator')) {
                        const headerDiv = item.querySelector('.process-header');
                        const indicatorDiv = document.createElement('div');
                        indicatorDiv.className = 'external-indicator';
                        indicatorDiv.innerHTML = '<i class="fa-solid fa-globe"></i>';
                        headerDiv.appendChild(indicatorDiv);
                    }
                }
            }
        });
    }

    isLocalhostIP(ip) {
        return ip === '127.0.0.1' || ip === 'localhost' || ip === '::1';
    }

    getIPType(ip) {
        const octets = ip.split('.');
        if (octets.length !== 4) return 'external';

        if (octets[0] === '10') return 'private';
        if (octets[0] === '172' && (parseInt(octets[1]) >= 16 && parseInt(octets[1]) <= 31)) return 'private';
        if (octets[0] === '192' && octets[1] === '168') return 'private';
        if (octets[0] === '169' && octets[1] === '254') return 'private';

        return 'external';
    }

    handleProcessClick(e) {
        const processItem = e.target.closest('.process-item');
        if (!processItem) return;

        const processId = processItem.dataset.processId;
        const process = this.processData[processId];

        let direction = null;

        if (process.outbound &&
            process.outbound.lastActivity &&
            !this.isLocalhostIP(process.outbound.dstIp) &&
            this.getIPType(process.outbound.dstIp) === 'external') {
            direction = 'outbound';
        }
        else if (process.outbound && process.outbound.lastActivity) {
            direction = 'outbound';
        }
        else if (process.inbound && process.inbound.lastActivity) {
            direction = 'inbound';
        }

        if (!direction) {
            console.log('No network activity found for process', processId);
            return;
        }

        document.querySelectorAll('.process-item').forEach(item => {
            item.classList.remove('active-process');
        });
        processItem.classList.add('active-process');

        this.activeProcessId = processId;
        this.activeDirection = direction;
        this.showProcessDetails(processId, direction);
    }

    createDetailRow(label, value) {
        const row = document.createElement('tr');

        const labelCell = document.createElement('td');
        labelCell.textContent = label + ':';

        const valueCell = document.createElement('td');
        valueCell.textContent = value;

        row.appendChild(labelCell);
        row.appendChild(valueCell);

        return row;
    }

    showProcessDetails(processId, direction) {
        const process = this.processData[processId];
        if (!process) return;

        const directionData = process[direction];
        if (!directionData || !directionData.lastActivity) return;

        document.getElementById('detail-title').textContent = `${process.name} (${direction})`;

        const processTables = document.querySelectorAll('.detail-table');
        processTables.forEach(table => {
            const processTable = table.closest('.detail-section:first-child');
            const rows = table.querySelectorAll('tr');

            if (processTable) {
                for (let i = 6; i < rows.length; i++) {
                    rows[i].remove();
                }
            } else {
                for (let i = 5; i < rows.length; i++) {
                    rows[i].remove();
                }
            }
        });

        document.getElementById('detail-process-name').textContent = process.name;
        document.getElementById('detail-pid').textContent = process.pid;
        document.getElementById('detail-owner').textContent = process.owner;
        document.getElementById('detail-memory').textContent = `${parseInt(process.memory).toLocaleString()} KB`;
        document.getElementById('detail-create-time').textContent = this.formatDate(new Date(process.createTime));
        const cmdlineElement = document.getElementById('detail-cmdline');
        cmdlineElement.textContent = process.cmdline;

        const processTable = document.querySelector('.detail-section:first-child .detail-table');

        if (process.parent) {
            const row = this.createDetailRow('Parent Process', `${process.parent.name} (PID: ${process.parent.pid})`);
            processTable.appendChild(row);
        }

        if (process.cpuPercent !== undefined) {
            const row = this.createDetailRow('CPU Usage', `${process.cpuPercent.toFixed(1)}%`);
            processTable.appendChild(row);
        }

        if (process.threadCount !== undefined) {
            const row = this.createDetailRow('Threads', process.threadCount);
            processTable.appendChild(row);
        }

        if (process.status) {
            const row = this.createDetailRow('Status', process.status);
            processTable.appendChild(row);
        }

        if (process.ioCounters) {
            const readMB = (process.ioCounters.readBytes / (1024 * 1024)).toFixed(2);
            const writeMB = (process.ioCounters.writeBytes / (1024 * 1024)).toFixed(2);
            const row = this.createDetailRow('Disk I/O', `Read: ${readMB} MB, Write: ${writeMB} MB`);
            processTable.appendChild(row);
        }

        cmdlineElement.onclick = function() {
            navigator.clipboard.writeText(process.cmdline).then(() => {
                const originalText = cmdlineElement.textContent;
                cmdlineElement.textContent = "Copied to clipboard!";
                cmdlineElement.style.color = "var(--success-color)";

                setTimeout(() => {
                    cmdlineElement.textContent = originalText;
                    cmdlineElement.style.color = "";
                }, 1500);
            });
        };

        document.getElementById('detail-direction').textContent = direction.charAt(0).toUpperCase() + direction.slice(1);
        document.getElementById('detail-last-activity').textContent = this.formatDate(new Date(directionData.lastActivity));
        document.getElementById('detail-src').textContent = `${directionData.srcIp}:${directionData.srcPort}`;
        document.getElementById('detail-dst').textContent = `${directionData.dstIp}:${directionData.dstPort}`;

        const networkTable = document.querySelector('.detail-section:last-child .detail-table');

        const ipToCheck = direction === 'outbound' ? directionData.dstIp : directionData.srcIp;
        const ipTypeRow = document.getElementById('ip-type-row');

        if (!this.isLocalhostIP(ipToCheck)) {
            const ipType = this.getIPType(ipToCheck);
            const typeCell = document.getElementById('ip-type');

            let ipTypeHtml = '';
            if (ipType === 'external') {
                ipTypeHtml = `<span class="reputation-badge warning">External IP</span>`;
            } else if (ipType === 'private') {
                ipTypeHtml = `<span class="reputation-badge info">Private Network</span>`;
            }

            typeCell.innerHTML = ipTypeHtml;
            ipTypeRow.classList.remove('hidden');
        } else {
            ipTypeRow.classList.add('hidden');
        }

        if (directionData.firstSeen) {
            const row = this.createDetailRow('Connection Start', this.formatDate(new Date(directionData.firstSeen)));
            networkTable.appendChild(row);
        }

        if (directionData.connectionDuration !== undefined) {
            const row = this.createDetailRow('Connection Duration', this.formatDuration(directionData.connectionDuration));
            networkTable.appendChild(row);
        }

        if (directionData.totalBytesSent !== undefined || directionData.totalBytesReceived !== undefined) {
            const sent = directionData.totalBytesSent ? this.formatBytes(directionData.totalBytesSent) : '0 B';
            const received = directionData.totalBytesReceived ? this.formatBytes(directionData.totalBytesReceived) : '0 B';

            const row = this.createDetailRow('Data Transfer', `Sent: ${sent}, Received: ${received}`);
            networkTable.appendChild(row);
        }
    }

    formatBytes(bytes) {
        if (bytes < 1024) {
            return `${bytes} B`;
        } else if (bytes < 1024 * 1024) {
            return `${(bytes / 1024).toFixed(1)} KB`;
        } else {
            return `${(bytes / (1024 * 1024)).toFixed(1)} MB`;
        }
    }

    formatDuration(seconds) {
        if (seconds < 60) {
            return `${seconds.toFixed(1)} sec`;
        } else if (seconds < 3600) {
            const mins = Math.floor(seconds / 60);
            const secs = Math.floor(seconds % 60);
            return `${mins}m ${secs}s`;
        } else {
            const hours = Math.floor(seconds / 3600);
            const mins = Math.floor((seconds % 3600) / 60);
            return `${hours}h ${mins}m`;
        }
    }

    formatTime(date) {
        if (!date || !(date instanceof Date) || isNaN(date)) return 'N/A';

        return date.toLocaleTimeString('en-US', {
            hour: '2-digit',
            minute: '2-digit',
            second: '2-digit',
            hour12: false
        });
    }

    formatDate(date) {
        if (!date || !(date instanceof Date) || isNaN(date)) return 'N/A';

        return date.toLocaleString('en-US', {
            year: 'numeric',
            month: '2-digit',
            day: '2-digit',
            hour: '2-digit',
            minute: '2-digit',
            second: '2-digit',
            hour12: false
        });
    }

    closeDetailPanel() {
        this.activeProcessId = null;
        this.activeDirection = null;

        document.querySelectorAll('.process-item').forEach(item => {
            item.classList.remove('active-process');
        });

        document.getElementById('detail-title').textContent = 'Process Details';
        document.getElementById('detail-process-name').textContent = '-';
        document.getElementById('detail-pid').textContent = '-';
        document.getElementById('detail-owner').textContent = '-';
        document.getElementById('detail-memory').textContent = '-';
        document.getElementById('detail-create-time').textContent = '-';
        document.getElementById('detail-cmdline').textContent = '-';
        document.getElementById('detail-direction').textContent = '-';
        document.getElementById('detail-last-activity').textContent = '-';
        document.getElementById('detail-src').textContent = '-';
        document.getElementById('detail-dst').textContent = '-';
        document.getElementById('ip-type-row').classList.add('hidden');
    }

    handleSearchInput() {
        const searchTerm = this.processSearch.value.toLowerCase();
        this.renderProcessList(searchTerm);
    }

    async exportData() {
        try {
            const response = await window.pywebview.api.export_data();

            if (response.status === 'success') {
                this.showAlert(`Data exported to ${response.filename} (CSV)`, 'success');
            } else {
                this.showAlert('Failed to export data', 'error');
            }
        } catch (error) {
            console.error('Export error:', error);
            this.showAlert(`Export failed: ${error.message}`, 'error');
        }
    }

    showAlert(message, type = 'success') {
        const existingAlert = document.querySelector('.alert-notification');
        if (existingAlert) {
            existingAlert.remove();
        }

        const alert = document.createElement('div');
        alert.className = `alert-notification ${type}`;

        const icon = type === 'success' ? 'fa-check-circle' : 'fa-exclamation-circle';

        alert.innerHTML = `
            <i class="fa-solid ${icon}"></i>
            <div>${message}</div>
        `;

        document.body.appendChild(alert);

        setTimeout(() => {
            alert.classList.add('show');
        }, 10);

        setTimeout(() => {
            alert.classList.remove('show');
            setTimeout(() => {
                alert.remove();
            }, 300);
        }, 3000);
    }

    renderProcessList(searchTerm = '') {
        if (searchTerm === undefined && this.processSearch) {
            searchTerm = this.processSearch.value.toLowerCase();
        } else {
            searchTerm = searchTerm || '';
        }

        const cacheKey = searchTerm + Object.keys(this.processData).length;

        if (this._processListCache && this._processListCache.key === cacheKey) {
            if (this.processList.innerHTML !== this._processListCache.html) {
                this.processList.innerHTML = this._processListCache.html;

                document.querySelectorAll('.process-item').forEach(item => {
                    item.addEventListener('click', (e) => this.handleProcessClick(e));
                });
            }
            return;
        }

        let html = '';

        const processIds = Object.keys(this.processData);
        if (processIds.length === 0) {
            html = `
                <div class="process-list-empty">
                    <i class="fa-solid fa-terminal"></i>
                    <p>No active processes</p>
                    <span>Click "Run" to begin monitoring</span>
                </div>
            `;
            this.processList.innerHTML = html;
            this._processListCache = { key: cacheKey, html };
            return;
        }

        let filteredIds = processIds.filter(id => {
            const process = this.processData[id];

            if (searchTerm) {
                return process.name.toLowerCase().includes(searchTerm.toLowerCase()) ||
                       process.pid.toString().includes(searchTerm);
            }

            return true;
        });

        const processesWithExternalConnections = {};
        filteredIds.forEach(id => {
            const process = this.processData[id];

            if (process.outbound &&
                process.outbound.lastActivity &&
                !this.isLocalhostIP(process.outbound.dstIp) &&
                this.getIPType(process.outbound.dstIp) === 'external') {
                    processesWithExternalConnections[id] = true;
            }
        });

        filteredIds.sort((a, b) => {
            const hasExternalA = processesWithExternalConnections[a] || false;
            const hasExternalB = processesWithExternalConnections[b] || false;

            if (hasExternalA && !hasExternalB) return -1;
            if (!hasExternalA && hasExternalB) return 1;

            const procA = this.processData[a];
            const procB = this.processData[b];

            const getLastActivity = (proc) => {
                const inboundTime = proc.inbound && proc.inbound.lastActivity
                    ? new Date(proc.inbound.lastActivity).getTime() : 0;
                const outboundTime = proc.outbound && proc.outbound.lastActivity
                    ? new Date(proc.outbound.lastActivity).getTime() : 0;
                return Math.max(inboundTime, outboundTime);
            };

            return getLastActivity(procB) - getLastActivity(procA);
        });

        if (filteredIds.length === 0) {
            html = `
                <div class="process-list-empty">
                    <i class="fa-solid fa-search"></i>
                    <p>No matching processes found</p>
                    <span>${searchTerm ? 'Try a different search term' : 'No processes detected'}</span>
                </div>
            `;
            this.processList.innerHTML = html;
            this._processListCache = { key: cacheKey, html };
            return;
        }

        filteredIds.forEach(id => {
            const process = this.processData[id];
            const isActive = id === this.activeProcessId ? 'active-process' : '';
            const hasExternalConnection = processesWithExternalConnections[id] ? 'external-connection' : '';

            let iconClass = 'fa-solid fa-microchip';
            if (/chrome|firefox|edge|safari|opera/i.test(process.name)) {
                iconClass = 'fa-solid fa-globe';
            } else if (/cmd|powershell|bash|terminal/i.test(process.name)) {
                iconClass = 'fa-solid fa-terminal';
            } else if (/explorer|finder/i.test(process.name)) {
                iconClass = 'fa-solid fa-folder';
            } else if (/notepad|word|excel|office|text/i.test(process.name)) {
                iconClass = 'fa-solid fa-file-alt';
            } else if (/python|java|node|npm|code/i.test(process.name)) {
                iconClass = 'fa-solid fa-code';
            } else if (/system|svchost|daemon|service/i.test(process.name)) {
                iconClass = 'fa-solid fa-cogs';
            }

            html += `
                <div class="process-item ${isActive} ${hasExternalConnection}" data-process-id="${id}">
                    <div class="process-header">
                        <div class="process-icon">
                            <i class="${iconClass}"></i>
                        </div>
                        <div class="process-title">
                            <h3 class="process-name">${process.name}</h3>
                            <span class="process-pid">PID: ${process.pid}</span>
                        </div>
                        ${hasExternalConnection ? '<div class="external-indicator"><i class="fa-solid fa-globe"></i></div>' : ''}
                    </div>
                </div>
            `;
        });

        this.processList.innerHTML = html;

        document.querySelectorAll('.process-item').forEach(item => {
            item.addEventListener('click', (e) => this.handleProcessClick(e));
        });

        this._processListCache = { key: cacheKey, html };
    }

    render() {
        this.renderProcessList();
    }
}